﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
	public class CommentLogic : ICommentLogic
	{
		private readonly ICommentStorage _projectStorage;
		public CommentLogic(ICommentStorage projectStorage)
		{
			_projectStorage = projectStorage;
		}
		public List<CommentViewModel>? ReadList(CommentSearchModel? model)
		{
			var list = model == null ? _projectStorage.GetFullList() : _projectStorage.GetFilteredList(model);
			if (list == null)
			{
				return null;
			}
			return list;
		}
		public CommentViewModel? ReadElement(CommentSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			var element = _projectStorage.GetElement(model);
			if (element == null)
			{
				return null;
			}
			return element;
		}
		public bool Create(CommentBindingModel model)
		{
			CheckModel(model);
			if (_projectStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Update(CommentBindingModel model)
		{
			CheckModel(model);
			if (_projectStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Delete(CommentBindingModel model)
		{
			CheckModel(model, false);
			if (_projectStorage.Delete(model) == null)
			{
				return false;
			}
			return true;
		}
		private void CheckModel(CommentBindingModel model, bool withParams =
	   true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.TEXT))
			{
				throw new ArgumentNullException("Нет текста комментария", nameof(model.TEXT));
			}
			var element = _projectStorage.GetElement(new CommentSearchModel
			{
				TEXT = model.TEXT
			});
			if (element != null && element.Id != model.Id && element.PerformerId == model.PerformerId && element.TaskId == model.TaskId)
			{
				throw new InvalidOperationException("Такой комментарий уже существует");
			}
		}
	}
}
