﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
	public class PerformerLogic : IPerformerLogic
	{
		private readonly IPerformerStorage _performerStorage;
		public PerformerLogic(IPerformerStorage performerStorage)
		{
			_performerStorage = performerStorage;
		}
		public List<PerformerViewModel>? ReadList(PerformerSearchModel? model)
		{
			var list = model == null ? _performerStorage.GetFullList() : _performerStorage.GetFilteredList(model);
			if (list == null)
			{
				return null;
			}
			return list;
		}
		public PerformerViewModel? ReadElement(PerformerSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			var element = _performerStorage.GetElement(model);
			if (element == null)
			{
				return null;
			}
			return element;
		}
		public bool Create(PerformerBindingModel model)
		{
			CheckModel(model);
			if (_performerStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Update(PerformerBindingModel model)
		{
			CheckModel(model);
			if (_performerStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Delete(PerformerBindingModel model)
		{
			CheckModel(model, false);
			if (_performerStorage.Delete(model) == null)
			{
				return false;
			}
			return true;
		}
		private void CheckModel(PerformerBindingModel model, bool withParams =
	   true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.NAME_PF))
			{
				throw new ArgumentNullException("Отсутствует имя",
			   nameof(model.NAME_PF));
			}
			if (string.IsNullOrEmpty(model.login))
			{
				throw new ArgumentNullException("Отсутствует логин",
			   nameof(model.login));
			}
			if (string.IsNullOrEmpty(model.password))
			{
				throw new ArgumentNullException("Отсутствует пароль",
			   nameof(model.password));
			}
		}
	}
}
