﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
	public class TagGroupLogic : ITagGroupLogic
	{
		private readonly ITagGroupStorage _projectStorage;
		public TagGroupLogic(ITagGroupStorage projectStorage)
		{
			_projectStorage = projectStorage;
		}
		public List<TagGroupViewModel>? ReadList(TagGroupSearchModel? model)
		{
			var list = model == null ? _projectStorage.GetFullList() : _projectStorage.GetFilteredList(model);
			if (list == null)
			{
				return null;
			}
			return list;
		}
		public TagGroupViewModel? ReadElement(TagGroupSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			var element = _projectStorage.GetElement(model);
			if (element == null)
			{
				return null;
			}
			return element;
		}
		public bool Create(TagGroupBindingModel model)
		{
			CheckModel(model);
			if (_projectStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Update(TagGroupBindingModel model)
		{
			CheckModel(model);
			if (_projectStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Delete(TagGroupBindingModel model)
		{
			CheckModel(model, false);
			if (_projectStorage.Delete(model) == null)
			{
				return false;
			}
			return true;
		}
		private void CheckModel(TagGroupBindingModel model, bool withParams =
	   true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.NAME_TG))
			{
				throw new ArgumentNullException("Нет названия",
			   nameof(model.NAME_TG));
			}
			var element = _projectStorage.GetElement(new TagGroupSearchModel
			{
				NAME_TG = model.NAME_TG
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("такой тег уже есть");
			}
		}
	}
}
