﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using DataModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
	public class TaskLogic : ITaskLogic
	{

		private readonly ITaskStorage _taskStorage;

		public TaskLogic(ITaskStorage taskStorage)
		{
			_taskStorage = taskStorage;
		}

		public bool CreateTask(TaskBindingModel model)
		{
			CheckModel(model);
			if (model.Status != MyTaskStatus.Неизвестен)
			{
				return false;
			}
			model.Status = MyTaskStatus.Новый;
			if (_taskStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}

        public List<TaskViewModel>? ReadList(TaskSearchModel? model)
        {
            var list = _taskStorage.GetFullList();
            if (list == null)
            {
                return null;
            }
            return list;
        }

        public bool TakeTaskInWork(TaskBindingModel model)
		{
			return ChangeStatus(model, MyTaskStatus.Выполняется);
		}

		public bool FinishTask(TaskBindingModel model)
		{
			return ChangeStatus(model, MyTaskStatus.Готово);
		}

		private void CheckModel(TaskBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
		}

		private bool ChangeStatus(TaskBindingModel model, MyTaskStatus newStatus)
		{
			CheckModel(model, false);
			var task = _taskStorage.GetElement(new TaskSearchModel { Id = model.Id });
			if (task == null)
			{
				return false;
			}
			if (task.Status + 1 != newStatus)
			{
				return false;
			}
			model.Status = newStatus;
			if (model.Status != MyTaskStatus.Готово && task.DEADLINE > DateTime.Now)
			{
				model.Status += 1;
			}
			if (_taskStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}
	}
}
