﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
	public class TeamLogic : ITeamLogic
	{
		private readonly ITeamStorage _projectStorage;
		public TeamLogic(ITeamStorage projectStorage)
		{
			_projectStorage = projectStorage;
		}
		public List<TeamViewModel>? ReadList(TeamSearchModel? model)
		{
			var list = model == null ? _projectStorage.GetFullList() : _projectStorage.GetFilteredList(model);
			if (list == null)
			{
				return null;
			}
			return list;
		}
		public TeamViewModel? ReadElement(TeamSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			var element = _projectStorage.GetElement(model);
			if (element == null)
			{
				return null;
			}
			return element;
		}
		public bool Create(TeamBindingModel model)
		{
			CheckModel(model);
			if (_projectStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Update(TeamBindingModel model)
		{
			CheckModel(model);
			if (_projectStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Delete(TeamBindingModel model)
		{
			CheckModel(model, false);
			if (_projectStorage.Delete(model) == null)
			{
				return false;
			}
			return true;
		}
		private void CheckModel(TeamBindingModel model, bool withParams =
	   true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.NAME_TM))
			{
				throw new ArgumentNullException("Нет названия",
			   nameof(model.NAME_TM));
			}
			var element = _projectStorage.GetElement(new TeamSearchModel
			{
				TeamName = model.NAME_TM
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("команда с таким названием уже есть");
			}
		}
	}
}
