﻿using ClientApp.Models;
using Microsoft.AspNetCore.Mvc;
using System.Diagnostics;
using Contracts.BindingModels;
using Contracts.ViewModels;
using DataModels;
using Microsoft.AspNetCore.Mvc.Rendering;

namespace ClientApp.Controllers
{
	public class HomeController : Controller
	{
		private readonly ILogger<HomeController> _logger;

		public HomeController(ILogger<HomeController> logger)
		{
			_logger = logger;
		}

        public IActionResult Index()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            // Передача имени пользователя в представление
            ViewData["UserName"] = APIClient.Client.NAME_PF;

            return View();
        }

        [HttpGet]
        public IActionResult Privacy()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.Client);
        }


        [HttpPost]
        public void Privacy(string login, string password, string fio)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
            {
                throw new Exception("Введите логин, пароль и ФИО");
            }
            APIClient.PostRequest("api/client/updatedata", new PerformerBindingModel
            {
                Id = APIClient.Client.Id,
                NAME_PF = fio,
                login = login,
                password = password
            });

            APIClient.Client.NAME_PF = fio;
            APIClient.Client.login = login;
            APIClient.Client.password = password;
            Response.Redirect("Index");
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }

        [HttpGet]
        public IActionResult Enter()
        {
            return View();
        }

        [HttpPost]
        public void Enter(string login, string password)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password))
            {
                throw new Exception("Введите логин и пароль");
            }
            APIClient.Client = APIClient.GetRequest<PerformerViewModel>($"api/client/login?login={login}&password={password}");
            if (APIClient.Client == null)
            {
                throw new Exception("Неверный логин/пароль");
            }
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Register()
        {
            return View();
        }

        [HttpPost]
        public void Register(string login, string password, string fio, string specialty)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio) || string.IsNullOrEmpty(specialty))
            {
                throw new Exception("Введите логин, пароль и ФИО");
            }
            APIClient.PostRequest("api/client/register", new PerformerBindingModel
            {
                NAME_PF = fio,
                login = login,
                password = password,
                SPECIALTY = specialty
            });
            Response.Redirect("Enter");
            return;
        }

        [HttpGet]
        public IActionResult CreateProject()
        {
            ViewBag.Teams =
            APIClient.GetRequest<List<TeamViewModel>>("api/main/getteamlist");
            return View();
        }
        [HttpPost]
		public void CreateProject(int Team, string name, string description)
		{
			if (APIClient.Client == null)
			{
				throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
			}
			APIClient.PostRequest("api/main/createproject", new ProjectBindingModel
			{
				TeamId = Team,
				NAME_PJ = name,
                DESCRIPTION_PJ = description
			});
			Response.Redirect("Index");
		}

        [HttpGet]
        public IActionResult CreateTask()
        {
            ViewBag.Projects =
            APIClient.GetRequest<List<ProjectViewModel>>("api/main/getprojectlist");
            ViewBag.TagGroups =
            APIClient.GetRequest<List<TagGroupViewModel>>("api/main/gettagslist");
            return View();
        }
        [HttpPost]
		public void CreateTask(string shortDesc, string description, DateTime deadline, int Project, int Tag)
		{
			if (APIClient.Client == null)
			{
				throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
			}
			APIClient.PostRequest("api/main/createtask", new TaskBindingModel
			{
				SHORT_DESC = shortDesc,
				DESCRIPTION = description,
				DEADLINE = deadline,
				ProjectId = Project,
				TagGroupId = Tag,
                Status = MyTaskStatus.Неизвестен
			});
			Response.Redirect("Index");
		}

        [HttpGet]
        public IActionResult CreateComment()
        {
            ViewBag.Tasks =
            APIClient.GetRequest<List<TaskViewModel>>("api/main/gettasklist");
            return View();
        }
        [HttpPost]
        public void CreateComment(int Task, string Text)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            APIClient.PostRequest("api/main/createcomment", new CommentBindingModel
            {
                PerformerId = APIClient.Client.Id,
                TaskId = Task,
                TEXT = Text
            });
            Response.Redirect("Index");
        }

        public IActionResult ViewComments()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<CommentViewModel>>($"api/main/getcomments?PerformerId={APIClient.Client.Id}"));
        }

        [HttpGet]
        public IActionResult ViewTasks()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }

            var tasks = APIClient.GetRequest<List<TaskViewModel>>($"api/main/gettasklist");

            var tasksForPerformer = tasks.Where(task => task.TaskPerformers.Any(tp => tp.Key == APIClient.Client.Id)).ToList();

            return View(tasksForPerformer);
        }

        [HttpGet]
        public IActionResult ViewProjects()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<ProjectViewModel>>("api/main/getprojectlist"));
        }

        [HttpPost]
		public void AssignPerformerToTask(int taskId, int performerId)
		{
			if (APIClient.Client == null)
			{
				throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
			}
			APIClient.PostRequest("api/main/assigntaskperformer", new TaskPerformerBindingModel
			{
				TaskId = taskId,
				PerformerId = performerId
			});
			Response.Redirect("Index");
		}
	}
}
