﻿using Contracts.BindingModels;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using DatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DatabaseImplement.Implements
{
	public class CommentStorage : ICommentStorage
	{
		public List<CommentViewModel> GetFullList()
		{
			using var context = new TaskTrackerDatabase();
			return context.Comments
					.Include(x => x.Task)
					.Include(x => x.Performer)
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public List<CommentViewModel> GetFilteredList(CommentSearchModel model)
		{
			using var context = new TaskTrackerDatabase();
			if (model.Id.HasValue)
			{
				return context.Comments
					.Include(x => x.Task)
					.Include(x => x.Performer)
					.Where(x => x.Id == model.Id)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			else if (model.PerformerId.HasValue)
			{
				return context.Comments
					.Include(x => x.Task)
					.Include(x => x.Performer)
					.Where(x => x.PerformerId == model.PerformerId)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			else if (model.TaskId.HasValue)
			{
				return context.Comments
					.Include(x => x.Task)
					.Include(x => x.Performer)
					.Where(x => x.TaskId == model.TaskId)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			return new();
		}

		public CommentViewModel? GetElement(CommentSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			return context.Comments
					.Include(x => x.Task)
					.Include(x => x.Performer)
					.FirstOrDefault(x => x.Id == model.Id)
					?.GetViewModel;
		}

		public CommentViewModel? Insert(CommentBindingModel model)
		{
			var newComment = Comment.Create(model);
			if (newComment == null)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			context.Comments.Add(newComment);
			context.SaveChanges();
			return context.Comments
					  .Include(x => x.Task)
					  .Include(x => x.Performer)
					  .FirstOrDefault(x => x.Id == newComment.Id)
					  ?.GetViewModel;
		}

		public CommentViewModel? Update(CommentBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var order = context.Comments.FirstOrDefault(x => x.Id == model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			context.SaveChanges();
			return context.Comments
						 .Include(x => x.Task)
						 .Include(x => x.Performer)
						 .FirstOrDefault(x => x.Id == model.Id)
						 ?.GetViewModel;
		}

		public CommentViewModel? Delete(CommentBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var element = context.Comments.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				var deletedElement = context.Comments
										   .Include(x => x.Task)
										   .Include(x => x.Performer)
										   .FirstOrDefault(x => x.Id == model.Id)
										   ?.GetViewModel;
				context.Comments.Remove(element);
				context.SaveChanges();
				return deletedElement;
			}
			return null;
		}
	}
}
