﻿using Contracts.BindingModels;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using DatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DatabaseImplement.Implements
{
	public class PerformerStorage : IPerformerStorage
	{
		public List<PerformerViewModel> GetFullList()
		{
			using var context = new TaskTrackerDatabase();
            return context.Performers
                    .Include(x => x.Team)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

		public List<PerformerViewModel> GetFilteredList(PerformerSearchModel model)
		{
			using var context = new TaskTrackerDatabase();
			if (model.Id.HasValue)
			{
				return context.Performers
					.Include(x => x.Team)
					.Where(x => x.Id == model.Id)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			else if (model.TeamId.HasValue)
			{
				return context.Performers
					.Include(x => x.Team)
					.Where(x => x.TeamId == model.TeamId)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			return new();
		}

        public PerformerViewModel? GetElement(PerformerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.login) && string.IsNullOrEmpty(model.password) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new TaskTrackerDatabase();
            return context.Performers
				.Include (x => x.Team)
				.FirstOrDefault(x =>
            (!string.IsNullOrEmpty(model.login) && x.login == model.login && !string.IsNullOrEmpty(model.password) && x.password == model.password)
            || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public PerformerViewModel? Insert(PerformerBindingModel model)
		{
			var newPerformer = Performer.Create(model);
			if (newPerformer == null)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			context.Performers.Add(newPerformer);
			context.SaveChanges();
			return context.Performers
					  .Include(x => x.Team)
					  .FirstOrDefault(x => x.Id == newPerformer.Id)
					  ?.GetViewModel;
		}

		public PerformerViewModel? Update(PerformerBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var order = context.Performers.FirstOrDefault(x => x.Id == model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			context.SaveChanges();
			return context.Performers
						 .Include(x => x.Team)
						 .FirstOrDefault(x => x.Id == model.Id)
						 ?.GetViewModel;
		}

		public PerformerViewModel? Delete(PerformerBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var element = context.Performers.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				var deletedElement = context.Performers
										   .Include(x => x.Team)
										   .FirstOrDefault(x => x.Id == model.Id)
										   ?.GetViewModel;
				context.Performers.Remove(element);
				context.SaveChanges();
				return deletedElement;
			}
			return null;
		}
	}
}
