﻿using Contracts.BindingModels;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using DatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DatabaseImplement.Implements
{
	public class ProjectStorage : IProjectStorage
	{
		public List<ProjectViewModel> GetFullList()
		{
			using var context = new TaskTrackerDatabase();
			return context.Projects
					.Include(x => x.Team)
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public List<ProjectViewModel> GetFilteredList(ProjectSearchModel model)
		{
			using var context = new TaskTrackerDatabase();
			if (model.Id.HasValue)
			{
				return context.Projects
					.Include(x => x.Team)
					.Where(x => x.Id == model.Id)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			else if (model.TeamId.HasValue)
			{
				return context.Projects
					.Include(x => x.Team)
					.Where(x => x.TeamId == model.TeamId)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			return new();
		}

		public ProjectViewModel? GetElement(ProjectSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			return context.Projects
					.Include(x => x.Team)
					.FirstOrDefault(x => x.Id == model.Id)
					?.GetViewModel;
		}

		public ProjectViewModel? Insert(ProjectBindingModel model)
		{
			var newProject = Project.Create(model);
			if (newProject == null)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			context.Projects.Add(newProject);
			context.SaveChanges();
			return context.Projects
					  .Include(x => x.Team)
					  .FirstOrDefault(x => x.Id == newProject.Id)
					  ?.GetViewModel;
		}

		public ProjectViewModel? Update(ProjectBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var order = context.Projects.FirstOrDefault(x => x.Id == model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			context.SaveChanges();
			return context.Projects
						 .Include(x => x.Team)
						 .FirstOrDefault(x => x.Id == model.Id)
						 ?.GetViewModel;
		}

		public ProjectViewModel? Delete(ProjectBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var element = context.Projects.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				var deletedElement = context.Projects
										   .Include(x => x.Team)
										   .FirstOrDefault(x => x.Id == model.Id)
										   ?.GetViewModel;
				context.Projects.Remove(element);
				context.SaveChanges();
				return deletedElement;
			}
			return null;
		}
	}
}
