﻿using Contracts.BindingModels;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using DatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DatabaseImplement.Implements
{
	public class TagGroupStorage : ITagGroupStorage
	{
		public List<TagGroupViewModel> GetFullList()
		{
			using var context = new TaskTrackerDatabase();
			return context.TagGroups
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public List<TagGroupViewModel> GetFilteredList(TagGroupSearchModel model)
		{
			using var context = new TaskTrackerDatabase();
			if (model.Id.HasValue)
			{
				return context.TagGroups
					.Where(x => x.Id == model.Id)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			return new();
		}

		public TagGroupViewModel? GetElement(TagGroupSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			return context.TagGroups
					.FirstOrDefault(x => x.Id == model.Id)
					?.GetViewModel;
		}

		public TagGroupViewModel? Insert(TagGroupBindingModel model)
		{
			var newTagGroup = TagGroup.Create(model);
			if (newTagGroup == null)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			context.TagGroups.Add(newTagGroup);
			context.SaveChanges();
			return context.TagGroups
					  .FirstOrDefault(x => x.Id == newTagGroup.Id)
					  ?.GetViewModel;
		}

		public TagGroupViewModel? Update(TagGroupBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var order = context.TagGroups.FirstOrDefault(x => x.Id == model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			context.SaveChanges();
			return context.TagGroups
						 .FirstOrDefault(x => x.Id == model.Id)
						 ?.GetViewModel;
		}

		public TagGroupViewModel? Delete(TagGroupBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var element = context.TagGroups.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				var deletedElement = context.TagGroups
										   .FirstOrDefault(x => x.Id == model.Id)
										   ?.GetViewModel;
				context.TagGroups.Remove(element);
				context.SaveChanges();
				return deletedElement;
			}
			return null;
		}
	}
}
