﻿using Contracts.BindingModels;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using Microsoft.EntityFrameworkCore;
using Task = DatabaseImplement.Models.Task;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DatabaseImplement.Implements
{
	public class TaskStorage : ITaskStorage
	{
		public List<TaskViewModel> GetFullList()
		{
			using var context = new TaskTrackerDatabase();
			return context.Tasks
				.Include(x => x.Project)
				.Include(x => x.TagGroup)
				.Include(x => x.Performers)
				.ThenInclude(x => x.Performer)
				.ToList()
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public TaskViewModel? GetElement(TaskSearchModel model)
		{
			if (string.IsNullOrEmpty(model.SHORT_DESC) && !model.Id.HasValue)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			return context.Tasks
				.Include(x => x.Project)
				.Include(x => x.TagGroup)
				.Include(x => x.Performers)
				.ThenInclude(x => x.Performer)
				.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
		}
		public TaskViewModel? Insert(TaskBindingModel model)
		{
            var newTask = Task.Create(model);
            if (newTask == null)
            {
                return null;
            }
            using var context = new TaskTrackerDatabase();
            context.Tasks.Add(newTask);
            context.SaveChanges();
            return context.Tasks
                      .Include(x => x.Project)
					  .Include(x => x.TagGroup)
					  .Include(x => x.Performers)
					  .ThenInclude(x => x.Performer)
                      .FirstOrDefault(x => x.Id == newTask.Id)
                      ?.GetViewModel;
        }
		public TaskViewModel? Update(TaskBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			using var transaction = context.Database.BeginTransaction();
			try
			{
				var reinforced = context.Tasks.FirstOrDefault(rec => rec.Id == model.Id);
				if (reinforced == null)
				{
					return null;
				}
				reinforced.Update(model);
				context.SaveChanges();
				reinforced.UpdatePerformers(context, model);
				transaction.Commit();
				return reinforced.GetViewModel;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
        }
		public TaskViewModel? Delete(TaskBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var element = context.Tasks
							.Include(x => x.Performers)
							.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
                var deletedElement = context.Tasks
                                      .Include(x => x.Project)
									  .Include(x => x.TagGroup)
									  .Include(x => x.Performers)
									  .ThenInclude(x => x.Performer)
                                      .FirstOrDefault(x => x.Id == model.Id)
                                      ?.GetViewModel;
                context.Tasks.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
			return null;
        }
	}
}
