﻿using Contracts.BindingModels;
using Contracts.SearchModels;
using Contracts.StoragesContracts;
using Contracts.ViewModels;
using DatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DatabaseImplement.Implements
{
	public class TeamStorage : ITeamStorage
	{
		public List<TeamViewModel> GetFullList()
		{
			using var context = new TaskTrackerDatabase();
			return context.Teams
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public List<TeamViewModel> GetFilteredList(TeamSearchModel model)
		{
			using var context = new TaskTrackerDatabase();
			if (model.Id.HasValue)
			{
				return context.Teams
					.Where(x => x.Id == model.Id)
					.Select(x => x.GetViewModel)
					.ToList();
			}
			return new();
		}

		public TeamViewModel? GetElement(TeamSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			return context.Teams
					.FirstOrDefault(x => x.Id == model.Id)
					?.GetViewModel;
		}

		public TeamViewModel? Insert(TeamBindingModel model)
		{
			var newTeam = Team.Create(model);
			if (newTeam == null)
			{
				return null;
			}
			using var context = new TaskTrackerDatabase();
			context.Teams.Add(newTeam);
			context.SaveChanges();
			return context.Teams
					  .FirstOrDefault(x => x.Id == newTeam.Id)
					  ?.GetViewModel;
		}

		public TeamViewModel? Update(TeamBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var order = context.Teams.FirstOrDefault(x => x.Id == model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			context.SaveChanges();
			return context.Teams
						 .FirstOrDefault(x => x.Id == model.Id)
						 ?.GetViewModel;
		}

		public TeamViewModel? Delete(TeamBindingModel model)
		{
			using var context = new TaskTrackerDatabase();
			var element = context.Teams.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				var deletedElement = context.Teams
										   .FirstOrDefault(x => x.Id == model.Id)
										   ?.GetViewModel;
				context.Teams.Remove(element);
				context.SaveChanges();
				return deletedElement;
			}
			return null;
		}
	}
}
