﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DataModels.Models;
using Contracts.BindingModels;
using Contracts.ViewModels;

namespace DatabaseImplement.Models
{
	public class Comment : ICommentModel
	{
		public int Id { get; private set; }

		[Required]
		public string TEXT { get; private set; } = string.Empty;

		[Required]
		public int TaskId { get; set; }
		public virtual Task Task { get; set; }

		[Required]
		public int PerformerId { get; set; }
		public virtual Performer Performer { get; set; }

		public static Comment? Create(CommentBindingModel model)
		{
			if (model == null)
			{
				return null;
			}
			return new()
			{
				Id = model.Id,
				TEXT = model.TEXT,
				PerformerId = model.PerformerId,
				TaskId = model.TaskId,
			};
		}

		public void Update(CommentBindingModel model)
		{
			if (model == null)
			{
				return;
			}
			TEXT = model.TEXT;
		}

		public CommentViewModel GetViewModel => new()
		{
			Id = Id,
			TEXT = TEXT,
			PerformerId = PerformerId,
			PerformerName = Performer.NAME_PF,
			TaskId = TaskId,
			TaskName = Task.SHORT_DESC
		};
	}
}
