﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Contracts.BindingModels;
using Contracts.ViewModels;
using DataModels;
using DataModels.Models;

namespace DatabaseImplement.Models
{
	public class Task : ITaskModel
	{
		public int Id { get; set; }

		[Required]
		public string SHORT_DESC { get; set; } = string.Empty;

		[Required]
		public string DESCRIPTION { get; set;} = string.Empty;
		public MyTaskStatus Status { get; set; }

		[Required]
		public DateTime DEADLINE { get; set; }

		[Required]
		public int ProjectId { get; set; }
		public virtual Project Project { get; set; }

		[Required]
		public int TagGroupId { get; set; }
		public virtual TagGroup TagGroup { get; set; }


		private Dictionary<int, string>? _taskPerformers = null;
		[NotMapped]
		public Dictionary<int, string> TaskPerformers
		{
			get
			{
				if (_taskPerformers == null)
				{
					_taskPerformers = Performers
					.ToDictionary(recPC => recPC.PerformerId, recPC => recPC.Performer.NAME_PF);
				}
				return _taskPerformers;
			}
		}
		[ForeignKey("TaskId")]
		public virtual List<TaskPerformers> Performers { get; set; } = new();


        public static Task? Create(TaskBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            using var context = new TaskTrackerDatabase();
            var task = new Task()
            {
                Id = model.Id,
                SHORT_DESC = model.SHORT_DESC,
                DESCRIPTION = model.DESCRIPTION,
                DEADLINE = model.DEADLINE,
                ProjectId = model.ProjectId,
                TagGroupId = model.TagGroupId,
                Performers = model.TaskPerformers.Select(x => new TaskPerformers
                {
                    Performer = context.Performers.First(y => y.Id == x.Key)
                }).ToList()
            };

            return task;
        }

        public void Update(TaskBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			SHORT_DESC = model.SHORT_DESC;
			DESCRIPTION = model.DESCRIPTION;
			DEADLINE = model.DEADLINE;
		}

		public TaskViewModel GetViewModel => new()
		{
			Id = Id,
			SHORT_DESC = SHORT_DESC,
			DESCRIPTION = DESCRIPTION,
			DEADLINE = DEADLINE,
			ProjectId = ProjectId,
			ProjectName = Project.NAME_PJ,
			TagGroupId = TagGroupId,
			TagName = TagGroup.NAME_TG,
			TaskPerformers = TaskPerformers
		};

		public void UpdatePerformers(TaskTrackerDatabase context, TaskBindingModel model)
		{
			var taskPerformers = context.TaskPerformers.Where(rec => rec.TaskId == model.Id).ToList();
			if (taskPerformers != null && taskPerformers.Count > 0)
			{ // удалили те, которых нет в модели
				context.TaskPerformers.RemoveRange(taskPerformers.Where(rec => !model.TaskPerformers.ContainsKey(rec.PerformerId)));
				context.SaveChanges();
				// обновили количество у существующих записей
				foreach (var updatePerformer in taskPerformers)
				{
					model.TaskPerformers.Remove(updatePerformer.PerformerId);
				}
				context.SaveChanges();
			}
			var task = context.Tasks.First(x => x.Id == Id);
			foreach (var rp in model.TaskPerformers)
			{
				context.TaskPerformers.Add(new TaskPerformers
				{
					Task = task,
					Performer = context.Performers.First(x => x.Id == rp.Key),
				});
				context.SaveChanges();
			}
			_taskPerformers = null;
		}
	}
}
