﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.ViewModels;
using Microsoft.AspNetCore.Mvc;
using DatabaseImplement.Models;
using DatabaseImplement;

namespace TaskTrackerRestApi.Controllers
{
	[Route("api/[controller]/[action]")]
	[ApiController]
	public class MainController : Controller
	{

		private readonly ITaskLogic _task;

		private readonly IProjectLogic _project;

		private readonly ICommentLogic _comment;

		private readonly ITeamLogic _team;

		private readonly ITagGroupLogic _tagGroup;

		public MainController(ITaskLogic task, IProjectLogic project, ICommentLogic comment, ITeamLogic team, ITagGroupLogic tagGroupLogic)
		{
			_task = task;
			_project = project;
			_comment = comment;
			_team = team;
			_tagGroup = tagGroupLogic;
		}

        [HttpGet]
        public List<TagGroupViewModel>? GetTagsList()
        {
            try
            {
                return _tagGroup.ReadList(null);
            }
            catch
            {
                throw;
            }
        }

        [HttpGet]
		public List<ProjectViewModel>? GetProjectList()
		{
			try
			{
				return _project.ReadList(null);
			}
			catch
			{
				throw;
			}
		}

        [HttpPost]
        public void CreateProject(ProjectBindingModel model)
        {
            try
            {
                _project.Create(model);
            }
            catch
            {
                throw;
            }
        }

        [HttpGet]
		public List<CommentViewModel>? GetCommentsList()
		{
			try
			{
				return _comment.ReadList(null);
			}
			catch
			{
				throw;
			}
		}

		[HttpGet]
		public List<CommentViewModel>? GetComments(int PerformerId)
		{
			try
			{
				return _comment.ReadList(new CommentSearchModel { PerformerId = PerformerId });
			}
			catch
			{
				throw;
			}
		}
        [HttpPost]
        public void CreateComment(CommentBindingModel model)
        {
            try
            {
                _comment.Create(model);
            }
            catch
            {
                throw;
            }
        }

        [HttpGet]
		public List<TeamViewModel>? GetTeamList()
		{
			try
			{
				return _team.ReadList(null);
			}
			catch
			{
				throw;
			}
		}

		[HttpGet]
		public List<TeamViewModel>? GetTeams(int teamId)
		{
			try
			{
				return _team.ReadList(new TeamSearchModel { Id = teamId });
			}
			catch
			{
				throw;
			}
		}
        [HttpPost]
        public void CreateTeam(TeamBindingModel model)
        {
            try
            {
                _team.Create(model);
            }
            catch
            {
                throw;
            }
        }

        [HttpGet]
        public List<TaskViewModel>? GetTaskList()
        {
            try
            {
                return _task.ReadList(null);
            }
            catch
            {
                throw;
            }
        }

        [HttpPost]
		public void CreateTask(TaskBindingModel model)
		{
			try
			{
				_task.CreateTask(model);
			}
			catch
			{
				throw;
			}
		}
	}
}
