
using BusinessLogic.BusinessLogic;
using Contracts.BusinessLogicContracts;
using Contracts.StoragesContracts;
using DatabaseImplement.Implements;
using Microsoft.AspNetCore.Identity;
using Microsoft.OpenApi.Models;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.
builder.Services.AddTransient<ICommentStorage, CommentStorage>();
builder.Services.AddTransient<IPerformerStorage, PerformerStorage>();
builder.Services.AddTransient<IProjectStorage, ProjectStorage>();
builder.Services.AddTransient<ITagGroupStorage, TagGroupStorage>();
builder.Services.AddTransient<ITaskStorage, TaskStorage>();
builder.Services.AddTransient<ITeamStorage, TeamStorage>();

builder.Services.AddTransient<ICommentLogic, CommentLogic>();
builder.Services.AddTransient<IPerformerLogic, PerformerLogic>();
builder.Services.AddTransient<IProjectLogic, ProjectLogic>();
builder.Services.AddTransient<ITagGroupLogic, TagGroupLogic>();
builder.Services.AddTransient<ITaskLogic, TaskLogic>();
builder.Services.AddTransient<ITeamLogic, TeamLogic>();

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
{
	c.SwaggerDoc("v1", new OpenApiInfo
	{
		Title = "TaskTrackerRestApi",
		Version
	= "v1"
	});
});

var app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
	app.UseSwagger();
	app.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json",
   "TaskTrackerRestApi v1"));
}

app.UseHttpsRedirection();

app.UseAuthorization();

app.MapControllers();

app.Run();
